import { Component, ChangeDetectionStrategy, ViewEncapsulation } from '@angular/core';
import { PrismClass } from './prism.class';
import { PrismService } from './prism.service';
export class PrismComponent extends PrismClass {
    constructor(prismService) {
        super(prismService);
        this.prismService = prismService;
        this.subscribe();
    }
    ngAfterViewInit() {
        this.prismService.highlightElement(this.codeElementRef);
    }
    ngOnChanges(changes) {
        this.onChanges(['code', 'language'], changes);
    }
    ngOnDestroy() {
        if (this.subscription) {
            if (this.subscription.code) {
                this.subscription.code.unsubscribe();
            }
            if (this.subscription.language) {
                this.subscription.language.unsubscribe();
            }
        }
    }
    subscribe() {
        this.subscription.language = this.prismService.subscribe('language', {
            next: language => {
                if (this.codeElementRef) {
                    if (this.change === true) {
                        this.prismService.highlightElement(this.codeElementRef);
                        this.change = false;
                    }
                }
            }
        });
        this.subscription.code = this.prismService.subscribe('code', {
            next: code => {
                if (this.codeElementRef) {
                    if (this.change === true) {
                        this.codeElementRef.nativeElement.innerHTML = code;
                        this.prismService.highlightElement(this.codeElementRef);
                        this.change = false;
                    }
                }
            }
        });
    }
}
PrismComponent.decorators = [
    { type: Component, args: [{
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                providers: [PrismService],
                selector: 'prism-highlight',
                template: `<pre class="language-{{language}}"><code #codeElementRef class="language-{{language}}" *ngIf="code">{{code}}</code><code #codeElementRef class="language-{{language}}" *ngIf="!code"><ng-content></ng-content></code></pre><ng-template *ngIf="code" #loading>Loading...</ng-template>`
            },] },
];
PrismComponent.ctorParameters = () => [
    { type: PrismService, },
];
//# sourceMappingURL=prism.component.js.map